default_config = {
    'backends': ['astra-python', 'astra-matlab'],
    'astra-matlab': {
        'root': None
    },
    'projection': dict(),
    'reconstruction': dict()
}
config = default_config
